package com.gic.enterprise.service.outer.impl;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.enterprise.constant.AccountStandardEnum;
import com.gic.enterprise.constant.Constants;
import com.gic.enterprise.dto.DictDTO;
import com.gic.enterprise.dto.SmsRecordDTO;
import com.gic.enterprise.entity.TabBillingAccount;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.enterprise.service.*;
import com.gic.mq.sdk.GicMQClient;
import com.gic.redis.data.util.RedisUtil;
import com.gic.store.utils.valid.ValidUtil;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.redisson.api.RAtomicLong;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-12-06 10:38
 */
@Service("smsRecordApiService")
public class SmsRecordApiServiceImpl implements SmsRecordApiService {

    private Logger logger = LogManager.getLogger(SmsRecordApiServiceImpl.class);

    @Autowired
    private BillingSmsRecordService billingSmsRecordService;
    @Autowired
    private AccountStandardService accountStandardService;
    @Autowired
    private BillingAccountService billingAccountService;
    @Autowired
    private BillingEnterpriseSmsPackageService billingEnterpriseSmsPackageService;
    @Autowired
    private DictApiService dictApiService;

    @Override
    public ServiceResponse<Void> verifySendSms(Integer enterpriseId, Integer sendCount) {
        if (enterpriseId == null || sendCount == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR);
        }
        RedissonClient redisClient = RedisUtil.getRedisClient();
        RAtomicLong smsCountR = redisClient.getAtomicLong(Constants.ENTERPRISE_BILLING_SMS_PACKAGE_COUNT + enterpriseId);
        long smsPackageCount = smsCountR.get();
        if (smsPackageCount > sendCount) {
            logger.info("enterpriseId:{}, 短信套餐包条数:{}, 发送条数：{}, 可以发送", enterpriseId, smsPackageCount, sendCount);
            return ServiceResponse.success();
        }

        long missCount = sendCount - smsPackageCount;

        TabBillingAccount billingAccount = billingAccountService.getByEnterpriseId(enterpriseId);
        Double accountBalance = billingAccount.getAccountBalance();
        Integer unitFee = accountStandardService.queryUnitFee(enterpriseId, AccountStandardEnum.SMS.getCode());

        double needFee = (double) (missCount * unitFee) / 1000;
        if (needFee - accountBalance > 5000d) {
            logger.info("enterpriseId:{}, 短信套餐包条数:{}, 余额：{}, 发送条数：{}, 不能发送", enterpriseId, smsPackageCount, accountBalance, sendCount);
            return EnterpriseServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "余额不足， 不能发送");
        } else {
            logger.info("enterpriseId:{}, 短信套餐包条数:{},余额：{}, 发送条数：{}, 可以发送", enterpriseId, smsPackageCount, accountBalance, sendCount);
            return EnterpriseServiceResponse.success();
        }
    }

    @Override
    public ServiceResponse<Void> saveSmsRecord(SmsRecordDTO smsRecordDTO) {
        ServiceResponse paramResult = ValidUtil.allCheckValidate(smsRecordDTO, SmsRecordDTO.SmsRecordValid.class);
        if (!paramResult.isSuccess()) {
            return paramResult;
        }
        GicMQClient client = GICMQClientUtil.getClientInstance();
        try {
            String json = JSON.toJSONString(smsRecordDTO);
            logger.info("收到国内短信计费信息：{}", json);
            client.sendMessage("smsMq", json);
//            this.run(json);
        } catch (Exception e) {
            logger.warn("调用mq失败", e);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> run(String arg) {
        // 缓存中计费
        GicMQClient client = GICMQClientUtil.getClientInstance();
        try {
            logger.info("国内短信缓存扣费：{}", arg);

            SmsRecordDTO smsRecordDTO = JSON.parseObject(arg, SmsRecordDTO.class);
            // 判断是扣套餐包还是扣余额
            RedissonClient redisClient = RedisUtil.getRedisClient();
            RAtomicLong smsCount = redisClient.getAtomicLong(Constants.ENTERPRISE_BILLING_SMS_PACKAGE_COUNT + smsRecordDTO.getEnterpriseId());

            smsRecordDTO.setCacheStatus(0);
            smsRecordDTO.setDatabaseStatus(0);

            // 缓存扣费 0套餐包  4余额
            Integer fee = accountStandardService.queryUnitFee(smsRecordDTO.getEnterpriseId(), AccountStandardEnum.SMS.getCode());
            int deductType = deduct(smsCount, smsRecordDTO.getEnterpriseId(), fee);
            smsRecordDTO.setPayType(deductType);
            smsRecordDTO.setFee((double)fee/1000);

            Integer smsRecordId = billingSmsRecordService.saveSmsRecord(smsRecordDTO);
            smsRecordDTO.setSmsRecordId(smsRecordId);

            // 队列
            client.sendMessage("smsDbMq", JSON.toJSONString(smsRecordDTO));
            billingSmsRecordService.updateCacheStatus(smsRecordDTO.getSmsRecordId());
//            this.executeDatabase(JSON.toJSONString(smsRecordDTO));
        } catch (Exception e) {
            logger.warn("调用mq失败", e);
        }
        return ServiceResponse.success();
    }

    private int deduct(RAtomicLong rSmsCount, Integer enterpriseId, Integer fee) {
        long smsCount = rSmsCount.get();
        if (smsCount > 0) {
            // 套餐包计费
            boolean smsCountDeduct = rSmsCount.compareAndSet(smsCount, smsCount - 1);
            if (!smsCountDeduct) {
                // 设置失败 重新计算
                // todo 这里不知道要不要加个延时之类的
                return deduct(rSmsCount, enterpriseId, fee);
            } else {
                return 0;
            }
        } else {
            // 账户计费
            billingAccountService.deductAccountByCache(fee.doubleValue() / 1000, enterpriseId);
            return 4;
        }
    }

    @Override
    public ServiceResponse<Void> executeDatabase(String arg) {
        logger.info("国内短信数据库扣费：{}", arg);
        SmsRecordDTO smsRecordDTO = JSON.parseObject(arg, SmsRecordDTO.class);
        smsRecordDTO.setDeductionTime(new Date());
        ServiceResponse<DictDTO> serviceDict = dictApiService.getDict(Constants.SMS_APP_TYPE, smsRecordDTO.getAppCode());
        if (serviceDict.isSuccess()) {
            DictDTO dictDTO = serviceDict.getResult();
            if (dictDTO != null) {
                String key = dictDTO.getKey();
                smsRecordDTO.setAppName(key);
            }
        }

        Integer deductType = smsRecordDTO.getPayType();
        if (deductType == 0) {
            // 短信套餐包扣费
            Integer enterpriseSmsPackageId = billingEnterpriseSmsPackageService.deduct(smsRecordDTO.getEnterpriseId());
            smsRecordDTO.setEnterpriseSmsPackageId(enterpriseSmsPackageId);
        } else if (deductType == 4) {
            // 余额扣费
            Integer unitFee = accountStandardService.queryUnitFee(smsRecordDTO.getEnterpriseId(), AccountStandardEnum.SMS.getCode());
            billingAccountService.deductAccountByDatabase(unitFee.doubleValue() / 1000, smsRecordDTO.getEnterpriseId());
        }
        billingSmsRecordService.updateDatabaseStatus(smsRecordDTO);
        return ServiceResponse.success();
    }
}
